/*____________________________________________________________________________*\
 *

 Copyright (c) 1997-2001 John Roy, Holger Zimmermann. All rights reserved.

 These sources, libraries and applications are
 FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 as long as the following conditions are adhered to.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer. 

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.

 3. Redistributions of any form whatsoever and all advertising materials 
    mentioning features must contain the following
    acknowledgment:
    "This product includes software developed by John Roy
    and Holger Zimmermann (http://pi3web.sourceforge.net/)."

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHORS OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 *____________________________________________________________________________*|
 *
 * $Source: /cvsroot/pi3web/Pi3Web_200/Source/Pi3API/HTTPUtil.h,v $
 * $Date: 2002/01/26 18:16:50 $
 *
 Description:
	HTTP utilities which require no context other than thier parameters.

\*____________________________________________________________________________*/
/* $SourceTop:$ */

#ifndef HTTPUTIL_H_
#define HTTPUTIL_H_

#include <time.h>

#include "Pi3API.h"

/*____________________________________________________________________________*\
 *
 Name:
	HTTPUtil_contractPath

 Synopsis:
	int HTTPUtil_contractPath( char *pPath )

 Description:
	Contract the directory path <B>pPath</B> in place by removing directories
	with the name '..'. 

 Notes:
 Return Values:
	Returns the number of directories above the root pPath which should be
	discarded. Typically a value of 0 is the desired result.

 Errors:
 See Also:
\*____________________________________________________________________________*/
PUBLIC_PIAPI int HTTPUtil_contractPath( char *pPath );

/*____________________________________________________________________________*\
 *
 Name:
	HTTPUtil_rFC822Time

 Synopsis:
	int HTTPUtil_rFC822Time( struct tm *pT, Pi3String *pResult )

 Description:
	This function converts a given time value into a valid RFC822 time string.

 Notes:
 Return Values:
	PIAPI_TRUE on success, PIAPI_ERROR otherwise. The converted time string is
	returned in pResult.

 Errors:
	PIAPI_ERROR

 See Also:
\*____________________________________________________________________________*/
PUBLIC_PIAPI int HTTPUtil_rFC822Time( struct tm *pT, Pi3String *pResult );

/*____________________________________________________________________________*\
 *
 Name:
	HTTPUtil_rcNumberToName

 Synopsis:
	const char *HTTPUtil_rcNumberToName( int iRC )

 Description:
	This function does convert a given PiAPI return code to the corresponding string.

 Notes:
 Return Values:
	The name corresponding to the given PiAPI return code
	<TABLE>
		<TR><TH>API return code<TH>Name string
		<TR><TD>PIAPI_COMPLETED<TD>COMPLETED
		<TR><TD>PIAPI_CONTINUE<TD>CONTINUE
		<TR><TD>PIAPI_ERROR<TD>ERROR
		<TR><TD>PIAPI_ABORT<TD>ABORT
		<TR><TD>INT_REDIRECT<TD>REDIRECT
		<TR><TD><TD>UNKNOWN
	</TABLE>

 Errors:
 See Also:
	HTTPUtil_rcNameToNumber
\*____________________________________________________________________________*/
PUBLIC_PIAPI const char *HTTPUtil_rcNumberToName( int iRC );

/*____________________________________________________________________________*\
 *
 Name:
	HTTPUtil_rcNameToNumber

 Synopsis:
	int HTTPUtil_rcNameToNumber( const char *pName )

 Description:
	This function returns the corresponding code to the name of a PiAPI return value.

 Notes:
 Return Values:
	Refer to the table of return values of HTTPUtil_rcNumberToName.

 Errors:
	PIAPI_ABORT is also returned on a wrong name in pName.

 See Also:
	HTTPUtil_rcNumberToName
\*____________________________________________________________________________*/
PUBLIC_PIAPI int HTTPUtil_rcNameToNumber( const char *pName );

/*____________________________________________________________________________*\
 *
 Name:
	HTTPUtil_phaseNumberToName

 Synopsis:
	const char *HTTPUtil_phaseNumberToName( int iPhaseNumber )

 Description:
	This function does convert a given process phase number to the corresponding string.

 Notes:
 Return Values:
	The name corresponding to the given process phase number
	<TABLE>
		<TR><TH>Phase No.<TH>Name string
		<TR><TD>PH_INVALID<TD>INVALID
		<TR><TD>PH_INIT<TD>INIT
		<TR><TD>PH_HEADERS<TD>HEADERS
		<TR><TD>PH_HOSTMAP<TD>HOSTMAP
		<TR><TD>PH_MAPPING<TD>MAPPING
		<TR><TD>PH_CHECKPATH<TD>CHECKPATH
		<TR><TD>PH_CHECKAUTH<TD>CHECKAUTH
		<TR><TD>PH_CHECKACCESS<TD>CHECKACCESS
		<TR><TD>PH_CHECKTYPE<TD>CHECKTYPE
		<TR><TD>PH_HANDLE<TD>HANDLE
		<TR><TD>PH_LOG<TD>LOG
		<TR><TD>PH_DESTROY<TD>DESTROY
	</TABLE>

 Errors:
 See Also:
	HTTPUtil_phaseNameToNumber
\*____________________________________________________________________________*/
PUBLIC_PIAPI const char *HTTPUtil_phaseNumberToName( int iPhaseNumber );

/*____________________________________________________________________________*\
 *
 Name:
	HTTPUtil_phaseNameToNumber

 Synopsis:
	int HTTPUtil_phaseNameToNumber( const char *pPhaseName )

 Description:
	This function returns the corresponding code to the given process phase name.

 Notes:
 Return Values:
	Refer to the table of return values of HTTPUtil_phaseNumberToName.

 Errors:
	PH_INVALID

 See Also:
	HTTPUtil_phaseNumberToName
\*____________________________________________________________________________*/
PUBLIC_PIAPI int HTTPUtil_phaseNameToNumber( const char *pPhaseName );

/*____________________________________________________________________________*\
 *
 Name:
	HTTPUtil_getHostName

 Synopsis:
	const char *HTTPUtil_getHostName( PIHTTP *pPIHTTP )

 Description:
	This function returns the name of the virtual host associated with the
	HTTP object in pPIHTTP.

 Notes:
 Return Values:
	The name of the virtual host or NULL.

 Errors:
 See Also:
	HTTPUtil_getHostPort
\*____________________________________________________________________________*/
PUBLIC_PIAPI const char *HTTPUtil_getHostName( PIHTTP *pPIHTTP );	

/*____________________________________________________________________________*\
 *
 Name:
	HTTPUtil_getHostPort

 Synopsis:
	const char *HTTPUtil_getHostPort( PIHTTP *pPIHTTP )

 Description:
	This function returns the port of the virtual host associated with the
	HTTP object in pPIHTTP.

 Notes:
 Return Values:
	The port number of the virtual host as decimal string or NULL.

 Errors:
 See Also:
	HTTPUtil_getHostName
\*____________________________________________________________________________*/
PUBLIC_PIAPI const char *HTTPUtil_getHostPort( PIHTTP *pPIHTTP );

/*____________________________________________________________________________*\
 *
 Name:
	HTTPUtil_protocolNumberToName

 Synopsis:
	const char *HTTPUtil_protocolNumberToName( int iProtocol )

 Description:
	This function does convert a given HTTP protocol version number to the
	corresponding string.

 Notes:
 Return Values:
	The name corresponding to the given process phase number
	<TABLE>
		<TR><TH>Protocol version<TH>Name string
		<TR><TD>PR_HTTP09<TD>HTTP/0.9
		<TR><TD>PR_HTTP10<TD>HTTP/1.0
		<TR><TD>PR_HTTP11<TD>HTTP/1.1
	</TABLE>

 Errors:
	The string "Unknown" is returned in case of unknown value in iProtocol.

 See Also:
\*____________________________________________________________________________*/
PUBLIC_PIAPI const char *HTTPUtil_protocolNumberToName( int iProtocol );

/*____________________________________________________________________________*\
 *
 Name:
	HTTPUtil_readTime

 Synopsis:
	int HTTPUtil_readTime( const char *pLine, struct tm *pT )

 Description:
	This function does scan pLine for a valid ANSI-C, RFC1123 or RFC850
	timestamp and parse it into a time structure.

 Notes:
	The pLine may contain more characters than only the time string.

 Return Values:
	PIAPI_COMPLETED on success, PIAPI_ERROR otherwise. The time structure
	in pT will be filled in case of success.

 Errors:
	PIAPI_ERROR

 See Also:
\*____________________________________________________________________________*/
PUBLIC_PIAPI int HTTPUtil_readTime( const char *pLine, struct tm *pT );

/*____________________________________________________________________________*\
 *
 Name:
	HTTPUtil_regexMatch

 Synopsis:
	int HTTPUtil_regexMatch( const char *pRegex, int iRegex, const char *pString,
		int iStringLen )

 Description:
	This function does match the regular expression in pRegex with length iRegex
	against iStringLen characters in pString.

 Notes:
	The regular expression syntax is currently very limitated, only asterisk '*'
	is supported, e.g. HTTPUtil_regexMatch("*admin*", 7, pPath, 25)" does scan
	the first 25 characters of pPath for pattern "admin" at any position.

 Return Values:
	PIAPI_TRUE if matched, PIAPI_FALSE otherwise

 Errors:
 See Also:
	HTTPUtil_regexMatchPi3String
\*____________________________________________________________________________*/
PUBLIC_PIAPI int HTTPUtil_regexMatch( const char *pRegex, int iRegex,
		const char *pString, int iStringLen );

/*____________________________________________________________________________*\
 *
 Name:
	HTTPUtil_regexMatchPi3String

 Synopsis:
	int HTTPUtil_regexMatchPi3String( const Pi3String *pRegexPattern,
		const Pi3String *pString )

 Description:
	This function does match the regular expression in Pi3String pRegexPattern
	against the Pi3String in pString.

 Notes:
 Return Values:
	PIAPI_TRUE if matched, PIAPI_FALSE otherwise

 Errors:
 See Also:
	HTTPUtil_regexMatch
\*____________________________________________________________________________*/
PUBLIC_PIAPI int HTTPUtil_regexMatchPi3String( const Pi3String *pRegexPattern,
		const Pi3String *pString ); 

/*____________________________________________________________________________*\
 *
 Name:
	HTTPUtil_urlEncode

 Synopsis:
	void HTTPUtil_urlEncode( const Pi3String *pToEncode, Pi3String *pResult )

 Description:
	This function does url-encode the string pToEncode in accordance with RFC1738.
	Spaces are additional swapped with '+'. 

 Notes:
 Return Values:
	The url-encoded string is returned in Pi3String pResult.

 Errors:
 See Also:
	HTTPUtil_urlDecode
\*____________________________________________________________________________*/
PUBLIC_PIAPI void HTTPUtil_urlEncode( const Pi3String *pToEncode,
		Pi3String *pResult );

/*____________________________________________________________________________*\
 *
 Name:
	HTTPUtil_urlDecode

 Synopsis:
	void HTTPUtil_urlDecode( const Pi3String *pToDecode, Pi3String *pResult )

 Description:
	This function does url-decode the string pToDecode in accordance with RFC1738,
	but with swapped spaces.

 Notes:
 Return Values:
	The url-decoded string is returned in Pi3String pResult.

 Errors:
 See Also:
	HTTPUtil_urlEncode
\*____________________________________________________________________________*/
PUBLIC_PIAPI void HTTPUtil_urlDecode( const Pi3String *pToDecode,
		Pi3String *pResult );

/*____________________________________________________________________________*\
 *
 Name:
	HTTPUtil_doHTTPError

 Synopsis:
	int HTTPUtil_doHTTPError( PIHTTP *pPIHTTP, int iError )

 Description:
	This function raises an HTTP error by
	<UL>
		<LI>set the error status code in pPIHTTP to iError
		<LI>clear any content-type, content-length, objectmap,
		authentication realm, authentication type in the databases
		<LI>reset the method to 'GET' in the request database
		<LI>disable keep alive
		<LI>cause an internal redirect
	</UL>
 Notes:
 Return Values:
	return INT_REDIRECT to cause a redirect

 Errors:
 See Also:
\*____________________________________________________________________________*/
PUBLIC_PIAPI int HTTPUtil_doHTTPError( PIHTTP *pPIHTTP, int iError );

/*____________________________________________________________________________*\
 *
 Name:
	HTTPUtil_sendFile

 Synopsis:
	HTTPUtil_sendFile( PIIOBuffer *pBuffer, PIFInfo *pFInfo,
		int iBufferingFlags, int iSendFlags )

 Description:
	This function sends the file resource as described by pFInfo through the PIIOBuffer
	to the client. The method doesn't consider HTTP headers, i.e. response headers must
	be sent before this method is invoked.

 Notes:
	For the iBufferingFlags refer to PIIOBuffer_write. The iSendFlags aren't used
	currently.

 Return Values:
	PIAPI_COMPLETED on success, PIAPI_ERROR otherwise.

 Errors:
	PIAPI_ERROR

 See Also:
	HTTPUtil_recvFile
	PIIOBuffer_write
\*____________________________________________________________________________*/
PUBLIC_PIAPI int HTTPUtil_sendFile( PIIOBuffer *pBuffer, PIFInfo *pFInfo,
		int iBufferingFlags, int iSendFlags );

/*____________________________________________________________________________*\
 *
 Name:
	HTTPUtil_recvFile

 Synopsis:
	HTTPUtil_recvFile( PIHTTP *pPIHTTP, PIIOBuffer *pBuffer, PIFInfo *pFInfo )

 Description:
	This function receives the file resource as described by pFInfo through the PIIOBuffer
	from the client. The method doesn't consider HTTP headers, i.e. request headers must be
	read before this method is invoked.

 Notes:
 Return Values:
	PIAPI_COMPLETED on success, PIAPI_ERROR otherwise.
 Errors:
	PIAPI_ERROR

 See Also:
	HTTPUtil_sendFile
	HTTPUtil_recvMultipartMsg
\*____________________________________________________________________________*/
PUBLIC_PIAPI int HTTPUtil_recvFile( PIHTTP *pPIHTTP, PIIOBuffer *pBuffer, PIFInfo *pFInfo );

/*____________________________________________________________________________*\
 *
 Name:
	HTTPUtil_doHTTPRequest

 Synopsis:
	int HTTPUtil_doHTTPRequest( PIIOBuffer *pBuffer, PIDB *pQ )

 Description:
	This function sends an HTTP request through pBuffer to the server,
	i.e. acts as an HTTP client. The request parameters are taken from the
	request database pQ. The function does particular:
	<UL>
		<LI>Gather items for the HTTP request and validate them
		<LI>Send the HTTP request line and standard headers 
		<LI>Send the other included RFC822 headers
	</UL>

 Notes:
	It is expected to find the HTTP method, the protocol version, the keep-alive flag and
	the request URI in pQ, otherwise PIAPI_EINVAL will be returned. A query string is considered
	as optional and appended to the URI.

 Return Values:
	PIAPI_COMPLETED on success, PIAPI_EINVAL on missing request parameters,
	PIAPI_ERROR otherwise.

 Errors:
	PIAPI_EINVAL
	PIAPI_ERROR

 See Also:

\*____________________________________________________________________________*/
PUBLIC_PIAPI int HTTPUtil_doHTTPRequest( PIIOBuffer *pBuffer,
    PIDB *pQ );

// callback prototype to handle multipart messages
typedef int ( *PFN_MUTLIPARTCB )( int iKind, char *szName, char *szData, long lData,
										    PIHTTP *pPIHTTP, void *pUser );
// callback types
#define MP_PART_NONE  0
#define MP_PART_FIRST 1
#define MP_PART_NEXT  2
#define MP_PART_LAST  4

/*____________________________________________________________________________*\
 *
Name:
	HTTPUtil_recvMultipartMsg

 Synopsis:
	HTTPUtil_recvMultipartMsg( PIHTTP *pPIHTTP, PIIOBuffer *pBuffer, PFN_MUTLIPARTCB cb, void *pUser )

 Description:
	This function is to be used to split a multipart HTTP message (RFC1867) into its parts. 

 Notes:
	The parameter 'cb' is a pointer to a callback function of type
	<PRE> 
	typedef int (__stdcall * PFN_MUTLIPARTCB )( int iKind,
                                                  char *szName,
                                                  char *szData,
                                                  long lData,
                                                  PIHTTP *pPIHTTP,
                                                  void *pUser );
	</PRE>

	The callback parameter 'kind' will be one ore more of the following constants

	<TABLE BORDER=1>
	<TR>
	<TH>Constant
	<TH>Description
	<TR>
	<TD>MP_PART_NONE
	<TD>Not used in callbacks.
	<TR>
	<TD>MP_PART_FIRST
	<TD>'szData' points to the first block of the message part with 'szName'.
	<TR>
	<TD>MP_PART_NEXT
	<TD>'szData' points to any block of the message part with 'szName'.
	<TR>
	<TD>MP_PART_LAST
	<TD>'szData' points to the last block of the message part with 'szName'.
	</TABLE>
	<P>
	The pointer 'pUser' is passed through by the callback function. It could point
	to any user defined data or class. If you make the callback function a friend
	of your handler class and 'pUser' is a reference to the handler you can use all
	class members of the handler in your callback function.

 Return Values:
	PIAPI_COMPLETED on success or PIAPI_ERROR on any error

 Errors:
	PIAPI_ERROR is returned if
	<UL>
	<LI>Content-Length header of the HTTP request is empty or 0
	<LI>The function couldn't allocate receive buffer
	<LI>The content isn't a multipart HTTP message (no header 'Content-Disposition' found)
	<LI>An error during read the data from the request occurs
	<LI>The user callback function returns PIAPI_ERROR
	</UL>

 See Also:
	HTTPUtil_recvFile
\*____________________________________________________________________________*/
PUBLIC_PIAPI int HTTPUtil_recvMultipartMsg( PIHTTP *pPIHTTP, PIIOBuffer *pBuffer, PFN_MUTLIPARTCB cb, void *pUser );

#endif /* HTTPUTIL_H_ */

